@@ -33,7 +33,7 @@ now_utc = datetime.now(pytz.utc)
 # Convert UTC time to Eastern Time
 eastern = pytz.timezone('US/Eastern')
 now_eastern = now_utc.astimezone(eastern)
-db = PolygonDatabase(host='localhost', user='chuck', database='markets', password='fud',port=5432)
+db = PolygonDatabase(host='localhost', user='chuck', database='market_data', password='fud',port=5432)
 # Format the datetime object to the desired format (YYYY-MM-DD HH:MM:SS)
 formatted_time = now_eastern.strftime('%Y-%m-%d %H:%M:%S')
 
