@@ -10,8 +10,8 @@ import pandas as pd
 import asyncio
 import requests
 from datetime import datetime, timezone, timedelta
-poly = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
-db = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
+poly = Polygon(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
+db = Polygon(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
 today = datetime.now().strftime('%Y-%m-%d')
 yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
 thirty_days_ago = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
@@ -878,8 +878,8 @@ class PolygonMaster:
     
 
 master = PolygonMaster()
-opts = PolygonOptions(user='chuck', database='markets', host='localhost', password='fud', port=5432)
-db = PolygonDatabase(host='localhost', user='chuck', database='markets', password='fud', port=5432)
+opts = PolygonOptions(user='chuck', database='market_data', host='localhost', password='fud', port=5432)
+db = PolygonDatabase(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
 async def main(ticker):
     ticker=ticker
     timespan='day'
