@@ -46,9 +46,9 @@ from apis.polygonio.async_polygon_sdk import Polygon
 from apis.polygonio.polygon_options import PolygonOptions
 class MultiMarkets(Inserts):
     def __init__(self, user, database, port, host, password):
-        self.poly = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
+        self.poly = Polygon(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
         self.discord = DiscordSDK()
-        self.feed_db = PolygonDatabase(host='localhost', user='chuck', port=5432, password='fud', database='markets')
+        self.feed_db = PolygonDatabase(host='localhost', user='chuck', port=5432, password='fud', database='market_data')
         self.technicals = Technicals()
         self.db = MarketDBManager(user=user,database=database,port=port,host=host,password=password)
         self.markets = [Market.Stocks, Market.Options,  Market.Forex] #Market.Forex]# Market.Indices]
@@ -76,7 +76,7 @@ class MultiMarkets(Inserts):
         self.indices_names=indices_names_and_symbols_dict
         self.agg_tickers = deque(maxlen=25)
         self.trade_tickers = deque(maxlen=250)
-        self.opts = PolygonOptions(user='chuck', database='markets', host='localhost', port=5432, password='fud')
+        self.opts = PolygonOptions(user='chuck', database='market_data', host='localhost', port=5432, password='fud')
         self.timespans = ['minute', 'hour', 'day', 'week', 'month']
         self.today = datetime.now().strftime('%Y-%m-%d')
         self.yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
@@ -580,7 +580,7 @@ class MultiMarkets(Inserts):
 
 
 
-market = MultiMarkets(host='localhost', user='chuck', database='markets', port=5432, password='fud')
+market = MultiMarkets(host='localhost', user='chuck', database='market_data', port=5432, password='fud')
 
 
 async def main():
