@@ -9,7 +9,7 @@ if project_dir not in sys.path:
     sys.path.append(project_dir)
 from fudstop.apis.webull.webull_options.webull_options import WebullOptions
 
-db = WebullOptions(database='markets', user='chuck')
+db = WebullOptions(database='market_data', user='chuck')
 
 
 
