@@ -39,7 +39,7 @@ class OptionsMarket:
         self.webull = WebullTrading()
         self.polygon = Polygon(database=database,user=user,password=password,port=port,host=host)
         self.poly_opts = PolygonOptions(user=user, database=database, host=host, port=port, password=password)
-        self.wb_opts = WebullOptions(user='chuck', database='markets')
+        self.wb_opts = WebullOptions(user='chuck', database='market_data')
         self.conn = None
         self.pool = None
         self.test = os.environ.get('test')
@@ -355,7 +355,7 @@ import logging
 logging.basicConfig(level=logging.INFO)
 
 
-options = OptionsMarket(database='markets')
+options = OptionsMarket(database='market_data')
 
 async def run_main_tasks(conn):
     num_workers = int(os.environ.get("NUM_WORKERS", 12))
