@@ -3,7 +3,7 @@ from dotenv import load_dotenv
 load_dotenv()
 from fudstop.apis.polygonio.async_polygon_sdk import Polygon
 from fudstop._markets.list_sets.ticker_lists import most_active_tickers
-polygon = Polygon(user='chuck', password='fud', host='localhost', port=5432, database='markets')
+polygon = Polygon(user='chuck', password='fud', host='localhost', port=5432, database='market_data')
 import aiohttp
 import json
 import asyncio
