@@ -1,7 +1,7 @@
 import os
 from dotenv import load_dotenv
 load_dotenv()
-from .models.application_commands import ApplicationCommands,Applications
+from .models.application_commands import ApplicationCommands,Applications, FUDSTOPBotCommands
 
 from .models.search import Attachments, Messages
 from . import DiscordDBManager
@@ -247,26 +247,22 @@ class DiscordSDK(DiscordDBManager):
         
 
 
-    def application_commands(self, guild):
+    async def application_commands(self):
         """
         >>> Returns: applications_df and commands_df
 
         >>> usage: applications, commands = application_commands(guild)
         
         """
-        url=f"https://discord.com/api/v9/guilds/{guild}/application-command-index"
+        url=f"https://discord.com/api/v9/guilds/1182826541063872522/application-commands/1233595915315581088"
         r = requests.get(url, headers=self.headers).json()
-        applications = Applications(r['applications'])
+
         application_commands = r['application_commands']
-        commands = ApplicationCommands(application_commands)
-
-        applications_df = applications.applications_as_dataframe
-        commands_df = commands.as_dataframe
 
 
+        return FUDSTOPBotCommands(application_commands)
 
-        return applications_df, commands_df
-
+            
 
     def get_webhooks(self, guild_id):
         url=f"https://discord.com/api/v9/guilds/{guild_id}/webhooks"
@@ -343,3 +339,11 @@ class DiscordSDK(DiscordDBManager):
 
         return upload_url
     
+
+    def create_poll(self, poll_question:str, channel_id:str, tts:bool, answer1, answer2, answer3):
+        payload={"mobile_network_type":"unknown","content": "","nonce":channel_id,"tts":tts,"flags":0,"poll":{"question":{"text":poll_question},"answers":[{"poll_media":{"text":answer1}},{"poll_media":{"text":answer2}},{"poll_media":{"text":answer3}}],"allow_multiselect":False,"duration":24,"layout_type":1}}
+        endpoint = f"https://discord.com/api/v9/channels/{channel_id}/messages"
+
+        r = requests.post(endpoint, data=json.dumps(payload), headers=self.headers).json()
+
+        print(r)
\ No newline at end of file
