@@ -13,13 +13,13 @@ YOUR_API_KEY = os.environ.get("YOUR_POLYGON_KEY")
 import re
 import pandas as pd
 
-
+import numpy as np
 from colorsys import rgb_to_hsv
 from bs4 import BeautifulSoup
 import requests
 from asyncio import Semaphore, TimeoutError
 from typing import List, Union, Any, Tuple, Dict
-from datetime import datetime
+from datetime import datetime, timezone
 import pytz
 import pandas as pd
 
@@ -53,7 +53,17 @@ def flatten_object(obj, parent_key='', separator='_'):
         else:
             items[new_key] = v
     return items
-
+def format_large_numbers_in_dataframe2(df, exclude_columns=[]):
+    """
+    Automatically formats all numeric columns in a DataFrame to readable large numbers,
+    excluding specified columns.
+    """
+    formatted_df = df.copy()
+    numeric_columns = df.select_dtypes(include=[np.number]).columns
+    for column in numeric_columns:
+        if column not in exclude_columns:
+            formatted_df[column] = formatted_df[column].apply(format_large_number)
+    return formatted_df
 def format_large_number(number):
     """
     Formats a number into a human-readable format (e.g., 1K, 1M, 1B, etc.) including negative numbers.
@@ -81,13 +91,16 @@ def format_large_numbers_in_dataframe(df):
     """
     Dynamically formats all numeric columns in a DataFrame to readable large numbers.
     """
-    formatted_df = df.copy()
-    numeric_columns = formatted_df.select_dtypes(include=['number']).columns
+    try:
+        formatted_df = df.copy()
+        numeric_columns = formatted_df.select_dtypes(include=['number']).columns
 
-    for column in numeric_columns:
-        formatted_df[column] = formatted_df[column].apply(format_large_number)
-    
-    return formatted_df
+        for column in numeric_columns:
+            formatted_df[column] = formatted_df[column].apply(format_large_number)
+        
+        return formatted_df
+    except Exception as e:
+        print(e)
 def chunk_data(data, chunk_size):
     """Yield successive chunk_size chunks from data."""
     for i in range(0, len(data), chunk_size):
@@ -218,7 +231,7 @@ def get_human_readable_string(string):
         strike_price = "380000"
 
     expiry_date = '20' + year + '-' + month + '-' + day
-    call_put = 'Call' if call_put == 'C' else 'Put'
+    call_put = 'call' if call_put == 'C' else 'put'
     strike_price = float(strike_price) / 1000
     result['underlying_symbol'] = underlying_symbol
     result['strike_price'] = strike_price
@@ -304,24 +317,20 @@ def convert_to_ns_datetime(unix_timestamp_str):
     dt_et = dt_et.replace(tzinfo=None)
     
     return dt_et   
-def convert_to_datetime(unix_timestamp_str):
-    # Convert Unix timestamp string to integer
-    unix_timestamp = int(unix_timestamp_str)
+def convert_to_eastern(timestamp_ms):
+    # Convert the timestamp from milliseconds to seconds
+    timestamp_sec = timestamp_ms / 1000
     
-    # Convert to datetime object in UTC
-    dt_utc = datetime.utcfromtimestamp(unix_timestamp)
+    # Create a timezone-aware UTC datetime object
+    utc_time = datetime.fromtimestamp(timestamp_sec, tz=timezone.utc)
     
-    # Localize to UTC
-    dt_utc = pytz.utc.localize(dt_utc)
-    
-    # Convert to Eastern Time
-    dt_et = dt_utc.astimezone(pytz.timezone('US/Eastern'))
-    
-    # Remove the timezone offset information, if you want to
-    dt_et = dt_et.replace(tzinfo=None)
+    # Convert to Eastern Time (ET)
+    eastern = pytz.timezone('US/Eastern')
+    eastern_time = utc_time.astimezone(eastern)
     
+    # Format without timezone info
+    return eastern_time.strftime('%Y-%m-%d %H:%M:%S')
 
-    return dt_et
 
 def convert_to_datetime_or_str(input_str):
     try:
@@ -767,7 +776,21 @@ async def paginate_tickers(url, as_dataframe=False, concurrency=5):
 
 
 
-
+# Example conversion function
+def convert_to_est(timestamp_str):
+    # Step 1: Parse the string into a datetime object (assuming UTC)
+    utc_time = datetime.strptime(timestamp_str, "%Y-%m-%dT%H:%M:%S.%fZ")
+    
+    # Step 2: Convert to EST
+    est_time = utc_time.replace(tzinfo=pytz.UTC).astimezone(pytz.timezone('US/Eastern'))
+    
+    # Step 3: Remove the timezone info
+    est_time = est_time.replace(tzinfo=None)
+    
+    # Step 4: Format the datetime object as a string
+    est_time_str = est_time.strftime("%Y-%m-%d %H:%M:%S")
+    
+    return est_time_str
 
 
 
