@@ -49,8 +49,8 @@ def is_etf(symbol):
 class MasterSDK:
     def __init__(self):
         self.trading = WebullTrading()
-        self.markets = WebullMarkets(host='localhost', database='markets', password='fud', port=5432, user='chuck')
-        self.occ = occSDK(host='localhost', database='markets', password='fud', port=5432, user='chuck')
+        self.markets = WebullMarkets(host='localhost', database='market_data', password='fud', port=5432, user='chuck')
+        self.occ = occSDK(host='localhost', database='market_data', password='fud', port=5432, user='chuck')
         self.fed = FedNewyork()
         self.fedprint = FedPrint()
         self.client = OpenAI(api_key=os.environ.get('OPENAI_KEY'))
@@ -506,7 +506,7 @@ class MasterSDK:
 
     async def ew_pivots(self, limit:int=20, insert:bool=False):
         """Get pivot point data for earnings. (support /resistance)"""
-        data = self.ew.pivot_list()
+        data = await self.ew.pivot_list()
 
         df = data.as_dataframe
         if insert == True:
@@ -516,7 +516,7 @@ class MasterSDK:
 
     async def ew_sentiment(self, limit:int=20, insert:bool=False):
         """Get earnings upcoming based on sentiment."""
-        data = self.ew.get_top_sentiment()
+        data = await self.ew.get_top_sentiment()
 
         df = data.as_dataframe
         if insert == True:
@@ -525,7 +525,7 @@ class MasterSDK:
     
     async def ew_upcoming_sector(self, limit:int=20, insert:bool=False):
         """Get earnings upcoming based on sentiment."""
-        data = self.ew.upcoming_sectors()
+        data = await self.ew.upcoming_sectors()
 
         df = data.as_dataframe
         if insert == True:
@@ -535,7 +535,7 @@ class MasterSDK:
 
     async def ew_upcoming_russell(self, limit:int=20, insert:bool=False):
         """Get earnings upcoming based on sentiment."""
-        data = self.ew.upcoming_russell()
+        data = await self.ew.upcoming_russell()
 
         df = data.as_dataframe
         if insert == True:
@@ -545,7 +545,7 @@ class MasterSDK:
 
     async def ew_today_results(self, limit:int=20, insert:bool=False):
         """Get earnings upcoming based on sentiment."""
-        data = self.ew.todays_results()
+        data = await self.ew.todays_results()
 
         df = data.as_dataframe
         if insert == True:
@@ -556,7 +556,7 @@ class MasterSDK:
         """Get earnings upcoming based on sentiment."""
         if date == None:
             date = self.thirty_days_from_now
-        data = self.ew.calendar(date=date)
+        data = await self.ew.calendar(date=date)
 
         df = data.as_dataframe
         if insert == True:
