@@ -16,7 +16,7 @@ from typing import List, Dict, Optional
 import pandas as pd
 import asyncio
 from aiohttp.client_exceptions import ClientConnectorError, ClientOSError, ClientConnectionError, ContentTypeError
-from fudstop.apis.webull.webull_trading import WebullTrading
+
 from .models.aggregates import Aggregates
 from .models.ticker_news import TickerNews
 from .models.company_info import CombinedCompanyResults
@@ -24,6 +24,8 @@ from .models.technicals import RSI, EMA, SMA, MACD
 from .models.gainers_losers import GainersLosers
 from .models.ticker_snapshot import StockSnapshot
 from .models.trades import TradeData, LastTradeData
+
+
 from datetime import datetime, timedelta
 import aiohttp
 
@@ -32,12 +34,24 @@ import requests
 from fudstop.apis.helpers import flatten_dict
 
 YOUR_POLYGON_KEY = os.environ.get('YOUR_POLYGON_KEY')
-
+todays_date = datetime.now().strftime('%Y-%m-%d')
+today = datetime.now().strftime('%Y-%m-%d')
+yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
+tomorrow = (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d')
+thirty_days_ago = (datetime.now() - timedelta(days=30)).strftime('%Y-%m-%d')
+two_days_ago = (datetime.now() - timedelta(days=2)).strftime('%Y-%m-%d')
+thirty_days_from_now = (datetime.now() + timedelta(days=30)).strftime('%Y-%m-%d')
+fifteen_days_ago = (datetime.now() - timedelta(days=15)).strftime('%Y-%m-%d')
+fifteen_days_from_now = (datetime.now() + timedelta(days=15)).strftime('%Y-%m-%d')
+eight_days_from_now = (datetime.now() + timedelta(days=8)).strftime('%Y-%m-%d')
+eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d') 
+ten_days_ago = (datetime.now() - timedelta(days=10)).strftime('%Y-%m-%d') 
 
 session = requests.session()
 class Polygon:
     def __init__(self, host='localhost', user='chuck', database='market_data', password='fud', port=5432):
         self.host=host
+        self.indices_list = ['NDX', 'RUT', 'SPX', 'VIX', 'XSP']
         self.port=port
         self.user=user
         self.password=password
@@ -52,12 +66,15 @@ class Polygon:
         self.fifteen_days_from_now = (datetime.now() + timedelta(days=15)).strftime('%Y-%m-%d')
         self.eight_days_from_now = (datetime.now() + timedelta(days=8)).strftime('%Y-%m-%d')
         self.eight_days_ago = (datetime.now() - timedelta(days=8)).strftime('%Y-%m-%d')
-        self.trading = WebullTrading()
+
         self.timeframes = ['minute', 'hour','day', 'week', 'month']
         self.session = None
     async def create_session(self):
-        if self.session is None or self.session.closed:
-            self.session = aiohttp.ClientSession()
+        try:
+            if self.session is None or self.session.closed:
+                self.session = aiohttp.ClientSession()
+        except Exception as e:
+            print(e)
 
     # Ensure to call this method to close the session
     async def close_session(self):
@@ -80,7 +97,7 @@ class Polygon:
                 port=os.environ.get('DB_PORT'),
                 user=os.environ.get('DB_USER'),
                 password=os.environ.get('DB_PASSWORD'),
-                database='polygon',
+                database='market_data',
                 min_size=1,
                 max_size=10
             )
@@ -219,7 +236,7 @@ class Polygon:
 
 
         endpoint = f"https://api.polygon.io/v2/aggs/ticker/{ticker}/range/{multiplier}/{timespan}/{date_from}/{date_to}?adjusted={adjusted}&sort={sort}&limit={limit}&apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
-
+  
         async with httpx.AsyncClient() as client:
             data = await client.get(endpoint)
 
@@ -234,8 +251,101 @@ class Polygon:
 
                 return results
 
+    async def fetch(self, url):
+
+        async with httpx.AsyncClient() as client:
+            resp = await client.get(url)
+
+            if resp.status_code == 200:
+                resp = resp.json()
+                return resp
+    async def fetch_realtime_price(self,ticker, multiplier:int=1, timespan:str='second', date_from:str=today, date_to:str=today):
+        if ticker in self.indices_list:
+            ticker = f"I:{ticker}"
+        url=f"https://api.polygon.io/v2/aggs/ticker/{ticker}/range/{multiplier}/{timespan}/{date_from}/{date_to}?sort=desc&apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
+
+        async with httpx.AsyncClient() as client:
+            data = await client.get(url)
+
+            if data.status_code == 200:
+                data = data.json()
+
+                results = data['results']
+                close = [i.get('c') for i in results]
+                close = close[0]
+
+                return close
+            
+    async def calculate_support_resistance(self, ticker, multiplier:int=1, timespan:str='hour', date_from:str=yesterday, date_to:str=today):
+        aggs = await self.get_aggs(ticker=ticker, multiplier=multiplier, timespan=timespan, date_from=date_from, date_to=date_to)
+        df = aggs.as_dataframe
+        
+        # Calculating Pivot Points, Support, and Resistance Levels
+        df['Pivot Point'] = (df['High'] + df['Low'] + df['Close']) / 3
+        df['Resistance Level'] = 2 * df['Pivot Point'] - df['Low']
+        df['Support Level'] = 2 * df['Pivot Point'] - df['High']
+        df['Stock Price'] = df['Close']
+        
+        # Selecting only the relevant columns
+        result_df = df[['Support Level', 'Resistance Level', 'Stock Price']]
+
+        return result_df
 
 
+
+    async def ema_check(self, ticker, ema_lengths):
+        """Checks jawless emas
+        
+        21/55/144 returns TRUE or FALSE if the EMAs are either above or below the current price."""
+     
+        # Define EMA lengths and corresponding URLs
+
+        urls = [
+            f"https://api.polygon.io/v1/indicators/ema/{ticker}?timespan=day&adjusted=true&window={length}&series_type=close&order=desc&apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
+            for length in ema_lengths
+        ]
+
+        # Fetch EMA data concurrently
+        tasks = [self.fetch(url) for url in urls]
+        data = await asyncio.gather(*tasks)
+
+        # Extract results and values, pairing them with EMA lengths
+        results = [i.get('results') for i in data]
+
+        # Convert EMA data to a DataFrame
+        values_per_ema = [
+            pd.DataFrame(
+                {
+                    "EMA Length": [length] * len(result.get('values', [])),
+                    "Date": [datetime.fromtimestamp(v['timestamp'] / 1000).strftime('%Y-%m-%d') for v in result.get('values', [])],
+                    "Value": [v['value'] for v in result.get('values', [])]
+                }
+            )
+            for length, result in zip(ema_lengths, results)
+        ]
+
+        # Concatenate all DataFrames into a single DataFrame
+        df = pd.concat(values_per_ema, ignore_index=True)
+        df['ticker'] = ticker
+
+        # Get the latest EMA values
+        latest_ema_values = df.sort_values('Date').groupby('EMA Length').tail(1).reset_index(drop=True)
+
+        # Fetch the current price
+        price = await self.get_price(ticker)
+
+        # Compare each EMA value with the current price
+        above_current_price = all(latest_ema_values['Value'] > price)
+        below_current_price = all(latest_ema_values['Value'] < price)
+
+        # Store the result as TRUE or FALSE string
+        all_above = "TRUE" if above_current_price else "FALSE"
+        all_below = "TRUE" if below_current_price else "FALSE"
+
+        df['above'] = all_above
+        df['below'] = all_below
+
+        return df
     async def market_news(self, limit: str = '100'):
         """
         Arguments:
@@ -453,18 +563,18 @@ class Polygon:
             
             endpoint = f"https://api.polygon.io/v1/indicators/rsi/{ticker}?timespan={timespan}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&window={window}&expand_underlying=true&apiKey={self.api_key}"
     
-
-            async with aiohttp.ClientSession() as session:
+       
+            async with httpx.AsyncClient() as client:
                 try:
-                    async with session.get(endpoint) as response:
-                        datas = await response.json()
+                    data = await client.get(endpoint)
+                    datas = data.json()
                       
-                        if datas is not None:
+                    if datas is not None:
 
-                            
+                        
 
-                    
-                            return RSI(datas, ticker)
+                
+                        return RSI(datas, ticker)
                 except ClientOSError as e:
                     print(f'ERROR {e}')
                 
@@ -479,17 +589,19 @@ class Polygon:
 
 
     async def get_cik(self, ticker):
+        try:
+            endpoint = f"https://api.polygon.io/v3/reference/tickers/{ticker}?apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
+            async with httpx.AsyncClient() as client:
+                data = await client.get(endpoint)
 
-        endpoint = f"https://api.polygon.io/v3/reference/tickers/{ticker}?apiKey={os.environ.get('YOUR_POLYGON_KEY')}"
-        async with httpx.AsyncClient() as client:
-            data = await client.get(endpoint)
+                cik = data.json()['results']['cik']
 
-            cik = data.json()['results']['cik']
+                
+                return cik
+        except Exception as e:
+            print(e)
 
-            
-            return cik
-
-    async def macd(self, ticker:str, timespan:str, limit:str='1000'):
+    async def macd(self, ticker: str, timespan: str, limit: str = '1000'):
         """
         Arguments:
 
@@ -508,28 +620,15 @@ class Polygon:
         >>> limit: the number of N timespans to survey
         
         """
-
-
-        
-        endpoint = f"https://api.polygon.io/v1/indicators/macd/{ticker}?timespan={timespan}&adjusted=true&short_window=12&long_window=26&signal_window=9&series_type=close&order=desc&apiKey={self.api_key}&limit={limit}"
-        await self.create_session()
         try:
-
-            try:
-                async with self.session.get(endpoint) as resp:
-                    datas = await resp.json()
-                    if datas is not None:
-
-                        
-  
-                
-                        return MACD(datas, ticker)
-            except (ClientConnectorError, ClientOSError, ContentTypeError):
-                print(f"ERROR - {ticker}")
-
+            endpoint = f"https://api.polygon.io/v1/indicators/macd/{ticker}?timespan={timespan}&adjusted=true&short_window=12&long_window=26&signal_window=9&series_type=close&order=desc&apiKey={self.api_key}&limit={limit}"
+            async with httpx.AsyncClient() as client:
+                data = await client.get(endpoint)
+                datas = data.json()
+                if datas is not None:
+                    return MACD(datas, ticker)
         except Exception as e:
-            print(e)
-
+            print(f"Unexpected error - {ticker}: {e}")
     async def sma(self, ticker:str, timespan:str, limit:str='1000', window:str='9', date_from:str=None, date_to:str=None):
         """
         Arguments:
@@ -552,26 +651,28 @@ class Polygon:
         >>> limit: the number of N timespans to survey
         
         """
-
-        if date_from is None:
-            date_from = self.eight_days_ago
-
-        if date_to is None:
-            date_to = self.today
-
-
-        endpoint = f"https://api.polygon.io/v1/indicators/sma/{ticker}?timespan={timespan}&window={window}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&apiKey={self.api_key}"
-        await self.create_session()
         try:
-            
+            if date_from is None:
+                date_from = self.eight_days_ago
 
-            async with self.session.get(endpoint) as resp:
-                datas = await resp.json()
+            if date_to is None:
+                date_to = self.today
 
 
-                return SMA(datas, ticker)
-        finally:
-            await self.close_session()
+            endpoint = f"https://api.polygon.io/v1/indicators/sma/{ticker}?timespan={timespan}&window={window}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&apiKey={self.api_key}"
+            await self.create_session()
+            try:
+                
+
+                async with self.session.get(endpoint) as resp:
+                    datas = await resp.json()
+
+
+                    return SMA(datas, ticker)
+            finally:
+                pass
+        except Exception as e:
+            print(e)
 
 
     async def ema(self, ticker:str, timespan:str, limit:str='1', window:str='21', date_from:str=None, date_to:str=None):
@@ -596,22 +697,24 @@ class Polygon:
         >>> limit: the number of N timespans to survey
         
         """
-
-        if date_from is None:
-            date_from = self.eight_days_ago
-
-        if date_to is None:
-            date_to = self.today
-
-
-        endpoint = f"https://api.polygon.io/v1/indicators/ema/{ticker}?timespan={timespan}&window={window}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&apiKey={self.api_key}"
-
-        
         try:
-            await self.create_session()  # Ensure the session is created
-            async with self.session.get(endpoint) as resp:
-                datas = await resp.json()
-                return EMA(datas, ticker)
+            if date_from is None:
+                date_from = self.eight_days_ago
+
+            if date_to is None:
+                date_to = self.today
+
+
+            endpoint = f"https://api.polygon.io/v1/indicators/ema/{ticker}?timespan={timespan}&window={window}&timestamp.gte={date_from}&timestamp.lte={date_to}&limit={limit}&apiKey={self.api_key}"
+
+            
+            try:
+                await self.create_session()  # Ensure the session is created
+                async with self.session.get(endpoint) as resp:
+                    datas = await resp.json()
+                    return EMA(datas, ticker)
+            except Exception as e:
+                print(e)
         except Exception as e:
             print(e)
 
@@ -652,62 +755,135 @@ class Polygon:
                     continue
             except Exception as e:
                 print(e)
-    async def get_price(self, ticker:str):
-        """
-        Fetches price from Webull API to use for option queries
 
-        Arguments:
 
-        >>> ticker: required - the ticker to survey
-        """
 
-        if ticker.startswith('I:'):
-            ticker = ticker.replace('I:', '')
-        datas = await self.trading.stock_quote(ticker)
-
-        price = datas.web_stock_close
-        if price is not None:
-            return float(price[0])
+    async def get_price(self, ticker):
+        try:
+            if ticker in ['SPX', 'NDX', 'XSP', 'RUT', 'VIX']:
+                ticker = f"I:{ticker}"
+            url = f"https://api.polygon.io/v3/snapshot?ticker.any_of={ticker}&limit=1&apiKey={self.api_key}"
+            print(url)
+            async with httpx.AsyncClient() as client:
+                r = await client.get(url)
+                if r.status_code == 200:
+                    r = r.json()
+                    results = r['results'] if 'results' in r else None
+                    if results is not None:
+                        session = [i.get('session') for i in results]
+                        price = [i.get('close') for i in session]
+                        return price[0]
+        except Exception as e:
+            print(f"{ticker} ... {e}")
 
     async def rsi_snapshot(self, ticker):
+        try:
+            price = await self.get_price(ticker)
+            if price is not None:
+                rsis = {}
+                rsis.update({'ticker': ticker,
+                            'price': price})
+                minrsi = asyncio.create_task(self.rsi(ticker,timespan='minute'))
+                drsi = asyncio.create_task(self.rsi(ticker, timespan='day'))
+                hrsi = asyncio.create_task(self.rsi(ticker, timespan='hour'))
+                wrsi = asyncio.create_task(self.rsi(ticker, timespan='week'))
+                mrsi = asyncio.create_task(self.rsi(ticker, timespan='month'))
+                day_rsi, hour_rsi, week_rsi, month_rsi, minute_rsi = await asyncio.gather(drsi, hrsi, wrsi, mrsi, minrsi)
+                if day_rsi is not None and hasattr(day_rsi, 'rsi_value') and day_rsi.rsi_value is not None and len(day_rsi.rsi_value) > 0:
+                    day_rsi = day_rsi.rsi_value[0]
 
-        price = await self.get_price(ticker)
-        if price is not None:
-            rsis = {}
-            rsis.update({'ticker': ticker,
-                        'price': price})
-            minrsi = asyncio.create_task(self.rsi(ticker,timespan='minute'))
-            drsi = asyncio.create_task(self.rsi(ticker, timespan='day'))
-            hrsi = asyncio.create_task(self.rsi(ticker, timespan='hour'))
-            wrsi = asyncio.create_task(self.rsi(ticker, timespan='week'))
-            mrsi = asyncio.create_task(self.rsi(ticker, timespan='month'))
-            day_rsi, hour_rsi, week_rsi, month_rsi, minute_rsi = await asyncio.gather(drsi, hrsi, wrsi, mrsi, minrsi)
-            if day_rsi is not None and hasattr(day_rsi, 'rsi_value') and day_rsi.rsi_value is not None and len(day_rsi.rsi_value) > 0:
-                day_rsi = day_rsi.rsi_value[0]
+                    rsis.update({'day_rsi': day_rsi})
 
-                rsis.update({'day_rsi': day_rsi})
-
-            if hour_rsi is not None and hasattr(hour_rsi, 'rsi_value') and hour_rsi.rsi_value is not None and len(hour_rsi.rsi_value) > 0:
-                hour_rsi = hour_rsi.rsi_value[0]
-                rsis.update({'hour_rsi': hour_rsi})
+                if hour_rsi is not None and hasattr(hour_rsi, 'rsi_value') and hour_rsi.rsi_value is not None and len(hour_rsi.rsi_value) > 0:
+                    hour_rsi = hour_rsi.rsi_value[0]
+                    rsis.update({'hour_rsi': hour_rsi})
 
 
-            if week_rsi is not None and hasattr(week_rsi, 'rsi_value') and week_rsi.rsi_value is not None and len(week_rsi.rsi_value) > 0:
-                week_rsi = week_rsi.rsi_value[0]
-                rsis.update({'week_rsi': week_rsi})
-            if month_rsi is not None and hasattr(month_rsi, 'rsi_value') and month_rsi.rsi_value is not None and len(month_rsi.rsi_value) > 0:
-                month_rsi = month_rsi.rsi_value[0]
-                rsis.update({'month_rsi': month_rsi})
+                if week_rsi is not None and hasattr(week_rsi, 'rsi_value') and week_rsi.rsi_value is not None and len(week_rsi.rsi_value) > 0:
+                    week_rsi = week_rsi.rsi_value[0]
+                    rsis.update({'week_rsi': week_rsi})
+                if month_rsi is not None and hasattr(month_rsi, 'rsi_value') and month_rsi.rsi_value is not None and len(month_rsi.rsi_value) > 0:
+                    month_rsi = month_rsi.rsi_value[0]
+                    rsis.update({'month_rsi': month_rsi})
 
 
-            if minute_rsi is not None and hasattr(minute_rsi, 'rsi_value') and minute_rsi.rsi_value is not None and len(minute_rsi.rsi_value) > 0:
-                minute_rsi = minute_rsi.rsi_value[0]
-                rsis.update({'minute_rsi': minute_rsi})
+                if minute_rsi is not None and hasattr(minute_rsi, 'rsi_value') and minute_rsi.rsi_value is not None and len(minute_rsi.rsi_value) > 0:
+                    minute_rsi = minute_rsi.rsi_value[0]
+                    rsis.update({'minute_rsi': minute_rsi})
 
-            
-            df = pd.DataFrame(rsis, index=[0])
+                
+                df = pd.DataFrame(rsis, index=[0])
+
+                return df
+        except Exception as e:
+            print(e)
+
+    async def check_macd_sentiment(self, hist: list):
+        try:
+            if hist is not None:
+                if hist is not None and len(hist) >= 3:
+                    
+                    last_three_values = hist[:3]
+                    if abs(last_three_values[0] - (-0.02)) < 0.04 and all(last_three_values[i] > last_three_values[i + 1] for i in range(len(last_three_values) - 1)):
+                        return 'bullish'
+
+                    if abs(last_three_values[0] - 0.02) < 0.04 and all(last_three_values[i] < last_three_values[i + 1] for i in range(len(last_three_values) - 1)):
+                        return 'bearish'
+                else:
+                    return '-'
+        except Exception as e:
+            print(e)
+
+
+
+    async def histogram_snapshot(self, ticker):
+        try:
+            price = await self.get_price(ticker)
+            if price is not None:
+                histograms = {}
+                dhist = asyncio.create_task(self.macd(ticker, timespan='day', limit='10'))
+                hhist= asyncio.create_task(self.macd(ticker, timespan='hour', limit='10'))
+                whist = asyncio.create_task(self.macd(ticker, timespan='week', limit='10'))
+                mhist = asyncio.create_task(self.macd(ticker, timespan='month', limit='10'))
+                day_hist, hour_hist, week_hist, month_hist = await asyncio.gather(dhist, hhist, whist, mhist)
+                if day_hist is not None and hasattr(day_hist, 'macd_histogram') and day_hist.macd_histogram is not None and len(day_hist.macd_histogram) > 2:
+                    day_hist = day_hist.macd_histogram
+         
+
+
+                    day_sentiment = await self.check_macd_sentiment(day_hist)
+                    histograms.update({'day_sentiment': day_sentiment})
+                if hour_hist is not None and hasattr(hour_hist, 'macd_histogram') and hour_hist.macd_histogram is not None and len(hour_hist.macd_histogram) > 2:
+                    hour_hist = hour_hist.macd_histogram
+                
+
+                    hour_sentiment = await self.check_macd_sentiment(hour_hist)
+                    histograms.update({'hour_sentiment': hour_sentiment})
+
+                if week_hist is not None and hasattr(week_hist, 'macd_histogram') and week_hist.macd_histogram is not None and len(week_hist.macd_histogram) > 2:
+                    week_hist = week_hist.macd_histogram
+          
+         
+
+                    week_sentiment = await self.check_macd_sentiment(week_hist)
+                    histograms.update({'week_sentiment': week_sentiment})
+                    
+                if month_hist is not None and hasattr(month_hist, 'macd_histogram') and month_hist.macd_histogram is not None and len(month_hist.macd_histogram) > 2:
+                    month_hist = month_hist.macd_histogram
+                
+
+
+                    month_sentiment = await self.check_macd_sentiment(month_hist)
+                    histograms.update({'month_sentiment': month_sentiment})
+                
+                df = pd.DataFrame(histograms, index=[0])
+                df['ticker'] = ticker
+                df['price'] = price
+
+                return df
+        except Exception as e:
+            print(e)
 
-            return df
 
     async def gather_rsi_for_all_tickers(self, tickers) -> List[RSI]:
 
@@ -755,27 +931,30 @@ class Polygon:
                 print(f"No logo found for symbol {symbol}")
             ```
         """
-        url = f'https://api.polygon.io/v3/reference/tickers/{symbol}?apiKey={self.api_key}'
-        await self.create_session()
         try:
-            async with self.session.get(url) as response:
-                data = await response.json()
-                
-                if 'results' not in data:
-                    # No results found
-                    return None
-                
-                results = data['results']
-                branding = results.get('branding')
+            url = f'https://api.polygon.io/v3/reference/tickers/{symbol}?apiKey={self.api_key}'
+            await self.create_session()
+            try:
+                async with self.session.get(url) as response:
+                    data = await response.json()
+                    
+                    if 'results' not in data:
+                        # No results found
+                        return None
+                    
+                    results = data['results']
+                    branding = results.get('branding')
 
-                if branding and 'icon_url' in branding:
-                    encoded_url = branding['icon_url']
-                    decoded_url = unquote(encoded_url)
-                    url_with_api_key = f"{decoded_url}?apiKey={self.api_key}"
-                    return url_with_api_key
+                    if branding and 'icon_url' in branding:
+                        encoded_url = branding['icon_url']
+                        decoded_url = unquote(encoded_url)
+                        url_with_api_key = f"{decoded_url}?apiKey={self.api_key}"
+                        return url_with_api_key
 
-        finally:
-            await self.close_session()
+            finally:
+                pass
+        except Exception as e:
+            print(e)
     async def stock_trades(self, ticker: str, limit: str = '50000', timestamp_gte: str = None, timestamp_lte: str = None):
         if timestamp_gte is None:
             timestamp_gte = self.thirty_days_ago
