@@ -1,16 +1,17 @@
 import sys
 from pathlib import Path
-
+import scipy.stats as stats
+import json
 # Add the project directory to the sys.path
 project_dir = str(Path(__file__).resolve().parents[1])
 if project_dir not in sys.path:
     sys.path.append(project_dir)
-
+import numpy as np
 import pandas as pd
 from datetime import datetime, timedelta
 from ...mapping import OPTIONS_EXCHANGES
 indices_list = ["SPX", "SPXW", "NDX", "VIX", "VVIX"]
-
+from scipy.stats import norm
 
 
 class UniversalSnapshot:
@@ -18,7 +19,7 @@ class UniversalSnapshot:
         self.today = datetime.now().strftime('%Y-%m-%d')
         self.yesterday = (datetime.now() - timedelta(days=1)).strftime('%Y-%m-%d')
         self.tomorrow = (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d')
-
+        self.risk_free_rate=5.37
         self.break_even_price = [i.get('session.break_even_price') for i in results]
         self.change = [i.get('session.change', None) for i in results]
         self.change_percent = [i.get('session.change_percent') for i in results]
@@ -205,7 +206,8 @@ class UniversalOptionSnapshot:
         self.break_even = [float(i['break_even_price']) if 'break_even_price' is not None and 'break_even_price' in i else None for i in results]
         self.implied_volatility = [float(i['implied_volatility']) if 'implied_volatility' in i else None for i in results] 
         self.open_interest = [float(i['open_interest']) if 'open_interest' in i else None for i in results]
-
+        self.risk_free_rate=5.37
+        
         day = [i['day'] if 'day' in i else 0 for i in results]
         self.volume = [float(i.get('volume',0)) for i in day]
         self.high = [float(i.get('high',0)) for i in day]
@@ -213,7 +215,7 @@ class UniversalOptionSnapshot:
         self.vwap = [float(i.get('vwap',0)) for i in day]
         self.open = [float(i.get('open',0)) for i in day]
         self.close = [float(i.get('close',0)) for i in day]
-        self.change_percent= [round(float(i.get('change_percent',0))*100,2) for i in day]
+        self.change_percent= [round(float(i.get('change_percent',0))) for i in day]
 
 
 
@@ -256,7 +258,7 @@ class UniversalOptionSnapshot:
         underlying_asset = [i['underlying_asset'] if i['underlying_asset'] is not None else None for i in results]
         self.change_to_breakeven = [float(i['change_to_break_even']) if 'change_to_break_even' in i else None for i in underlying_asset]
         self.underlying_price = [float(i.get('price')) if i.get('price') is not None else None for i in underlying_asset]
-
+        self.risk_free_rate = [self.risk_free_rate] * len(self.underlying_price)
         self.underlying_ticker = [i['ticker'] if 'ticker' in i else None for i in underlying_asset]
         today = pd.Timestamp(datetime.today())
         
@@ -326,17 +328,70 @@ class UniversalOptionSnapshot:
         iv_series = pd.Series(self.implied_volatility).dropna()
         self.iv_percentile = [round(x, 2) for x in iv_series.rank(pct=True)]
 
-        # Add additional second and third order Greeks
-        # Add additional second and third order Greeks
-        self.vanna = [(v * d) / 100 if v is not None and d is not None else None for v, d in zip(self.vega, self.delta)]
-        self.vomma = [(v * (d * (1 - d))) / 10000 if v is not None and d is not None else None for v, d in zip(self.vega, self.delta)]
-        self.charm = [(d * t) / 365 if d is not None and t is not None else None for d, t in zip(self.delta, self.days_to_expiry_series)]
-        self.veta = [(v * (d - 0.5)) if v is not None and d is not None else None for v, d in zip(self.vega, self.delta)]
-        self.speed = [(g * d) / 100 if g is not None and d is not None else None for g, d in zip(self.gamma, self.delta)]
-        self.zomma = [(g * (d - 0.5)) if g is not None and d is not None else None for g, d in zip(self.gamma, self.delta)]
-        self.color = [(g * t) / 365 if g is not None and t is not None else None for g, t in zip(self.gamma, self.days_to_expiry_series)]
-        self.ultima = [(v * (3 * d ** 2 - 2 * d + 1)) / 1000000 if v is not None and d is not None else None for v, d in zip(self.vega, self.delta)]
+        # Vanna Calculations (with validation for iv, u, d)
+        self.vanna = [(v * d / u) if v is not None and d is not None and u is not None and u > 0 else None 
+                    for v, d, u in zip(self.vega, self.delta, self.underlying_price)]
 
+        self.vanna_vega = [(d * (v / u)) if d is not None and v is not None and u is not None and u > 0 else None 
+                        for d, v, u in zip(self.delta, self.vega, self.underlying_price)]
+
+        self.vanna_delta = [(d / iv) if d is not None and iv is not None and iv > 0 else None 
+                            for d, iv in zip(self.delta, self.implied_volatility)]
+
+
+        # d1 Calculation (with validation for u, s, r, iv, t)
+        d1 = [(np.log(u / s) + (r + (iv**2) / 2) * t) / (iv * np.sqrt(t)) if u is not None and u > 0 and s is not None and s > 0 and r is not None and iv is not None and iv > 0 and t > 0 else None 
+            for u, s, r, iv, t in zip(self.underlying_price, self.strike, self.risk_free_rate, self.implied_volatility, self.days_to_expiry_series / 365)]
+
+        # Color d1 Calculation (with validation)
+        color_d1 = [(np.log(u / s) + (r + (iv**2) / 2) * (t / 365)) / (iv * np.sqrt(t / 365)) if u is not None and u > 0 and s is not None and s > 0 and r is not None and iv is not None and iv > 0 and t > 0 else None 
+                    for u, s, r, iv, t in zip(self.underlying_price, self.strike, self.risk_free_rate, self.implied_volatility, self.days_to_expiry_series)]
+
+
+        # Color Calculation (with validation for iv, t)
+        self.color = [(g / (2 * t) * (1 + (r - 0.5 * iv**2) / (iv * np.sqrt(t)) * d1_val)) if g is not None and g > 0 and t > 0 and d1_val is not None and iv is not None and iv > 0 else None 
+                    for g, t, r, iv, d1_val in zip(self.gamma, self.days_to_expiry_series / 365, self.risk_free_rate, self.implied_volatility, d1)]
+
+
+
+        Nd1_prime = [norm.pdf(d) if d is not None else None for d in d1]
+
+        # Charm Calculation (with validation for iv, t)
+        self.charm = [((-nd1p * (r / (iv * np.sqrt(t))) - d * r / t)) if d is not None and nd1p is not None and r is not None and iv is not None and iv > 0 and t > 0 else None 
+                    for nd1p, d, r, iv, t in zip(Nd1_prime, self.delta, self.risk_free_rate, self.implied_volatility, self.days_to_expiry_series / 365)]
+
+
+        # Veta Calculation (with validation for u, t)
+        self.veta = [-(u * nd1p * 0.5 / np.sqrt(t)) if u is not None and u > 0 and nd1p is not None and t > 0 else None 
+                    for u, nd1p, t in zip(self.underlying_price, Nd1_prime, self.days_to_expiry_series / 365)]
+
+
+        # Zomma Calculation (with validation for iv, u, d1)
+        self.zomma = [(g * (d1_val / (iv * u))) if g is not None and g > 0 and d1_val is not None and iv is not None and iv > 0 and u is not None and u > 0 else None 
+                    for g, d1_val, iv, u in zip(self.gamma, d1, self.implied_volatility, self.underlying_price)]
+
+        
+        # Speed Calculation (with input validation and edge case handling)
+        self.speed = [
+            (g * (2 * d)) / (u * iv * np.sqrt(t + 1e-6)) 
+            if g is not None and d is not None and u is not None and u > 0 and iv is not None and iv > 0 and t > 0 
+            else None 
+            for g, d, u, iv, t in zip(self.gamma, d1, self.underlying_price, self.implied_volatility, self.days_to_expiry_series / 365)
+        ]
+
+        # Ultima Calculation (with input validation and edge case handling)
+        self.ultima = [
+            (v * (3 * d1**2 - 1) * u**2) / (max(iv**2, 1e-6)) 
+            if v is not None and u is not None and u > 0 and iv is not None and iv > 0 and d1 is not None 
+            else None 
+            for v, u, iv, d1 in zip(self.vega, self.underlying_price, self.implied_volatility, d1)
+        ]
+
+
+        self.vomma = [
+            (v * u**2 * d1 / max(iv, 1e-6)) if v is not None and d1 is not None and u is not None and iv is not None else None 
+            for v, d1, u, iv in zip(self.vega, d1, self.underlying_price, self.implied_volatility)
+        ]
         self.data_dict = {
             'strike': self.strike,
             'expiry': self.expiry,
@@ -388,6 +443,8 @@ class UniversalOptionSnapshot:
             'greeks_balance': self.gbs,
             'opp': self.opp,
             'vanna': self.vanna,
+            'vanna_delta': self.vanna_delta,
+            'vanna_vega': self.vanna_vega,
             'vomma': self.vomma,
             'charm': self.charm,
             'veta': self.veta,
@@ -397,55 +454,225 @@ class UniversalOptionSnapshot:
             'ultima': self.ultima
         }
 
+                # Create DataFrame from data_dict
         # Create DataFrame from data_dict
         self.df = pd.DataFrame(self.data_dict)
 
-        def __repr__(self) -> str:
-            return f"UniversalOptionSnapshot(break_even={self.break_even}, \
-                    implied_volatility={self.implied_volatility},\
-                    open_interest ={self.open_interest}, \
-                    change={self.exchange}, \
-                    expiry={self.expiry}, \
-                    ticker={self.ticker} \
-                    contract_type={self.contract_type}, \
-                    exercise_style={self.exercise_style}, \
-                    theta={self.theta}, \
-                    delta={self.delta}, \
-                    gamma={self.gamma}, \
-                    vega={self.vega}, \
-                    sip_timestamp={self.sip_timestamp}, \
-                    conditions={self.conditions}, \
-                    trade_price={self.trade_price}, \
-                    trade_size={self.trade_size}, \
-                    exchange={self.exchange}, \
-                    ask={self.ask}, \
-                    bid={self.bid}, \
-                    bid_size={self.bid_size}, \
-                    ask_size={self.ask_size}, \
-                    midpoint={self.midpoint}, \
-                    change_to_breakeven={self.change_to_breakeven}, \
-                    underlying_price={self.underlying_price}, \
-                    underlying_ticker={self.underlying_ticker})"
+        # Calculate weighted Greeks by OI for calls and puts
+        self.df['weighted_delta'] = self.df['delta'] * self.df['oi']
+        self.df['weighted_gamma'] = self.df['gamma'] * self.df['oi']
+        self.df['weighted_vega'] = self.df['vega'] * self.df['oi']
+        self.df['weighted_theta'] = self.df['theta'] * self.df['oi']
+        self.df['weighted_vanna'] = self.df['vanna'] * self.df['oi']
+        self.df['weighted_charm'] = self.df['charm'] * self.df['oi']
+        self.df['weighted_veta'] = self.df['veta'] * self.df['oi']
+        self.df['weighted_speed'] = self.df['speed'] * self.df['oi']
+        self.df['weighted_zomma'] = self.df['zomma'] * self.df['oi']
+        self.df['weighted_color'] = self.df['color'] * self.df['oi']
+        self.df['weighted_ultima'] = self.df['ultima'] * self.df['oi']
 
-        def __getitem__(self, index):
-            return self.df[index]
+        # Introduce weighted_oi as a composite metric
+        self.df['weighted_oi'] = (
+            self.df['weighted_delta'] +
+            self.df['weighted_gamma'] +
+            self.df['weighted_vega'] +
+            self.df['weighted_theta'] +
+            self.df['weighted_vanna'] +
+            self.df['weighted_charm'] +
+            self.df['weighted_veta'] +
+            self.df['weighted_speed'] +
+            self.df['weighted_zomma'] +
+            self.df['weighted_color'] +
+            self.df['weighted_ultima']
+        )
 
-        def __setitem__(self, index, value):
-            self.df[index] = value
+        # Calculate the relative position of the option's strike price to the current underlying price
+        self.df['strike_distance'] = np.abs(self.df['strike'] - self.df['underlying_price'])
 
-        def __iter__(self):
-            # If df is a DataFrame, it's already iterable (over its column labels)
-            # To iterate over rows, use itertuples or iterrows
-            self.iter = self.df.itertuples()
-            return self
+        # Adjust the momentum factor calculation
+        self.df['momentum_factor'] = self.df['strike_distance'].apply(lambda x: 1 / x if x > 0 else 1e6)
 
-        def __next__(self):
-            # Just return the next value from the DataFrame iterator
-            try:
-                return next(self.iter)
-            except StopIteration:
-                # When there are no more rows, stop iteration
-                raise StopIteration
+        # Calculate momentum-adjusted Greeks using the momentum factor derived from strike distance
+        self.df['momentum_delta'] = self.df['delta'] * self.df['momentum_factor']
+        self.df['momentum_gamma'] = self.df['gamma'] * self.df['momentum_factor']
+        self.df['momentum_vega'] = self.df['vega'] * self.df['momentum_factor']
+        self.df['momentum_theta'] = self.df['theta'] * self.df['momentum_factor']
+        self.df['momentum_vanna'] = self.df['vanna'] * self.df['momentum_factor']
+        self.df['momentum_charm'] = self.df['charm'] * self.df['momentum_factor']
+        self.df['momentum_veta'] = self.df['veta'] * self.df['momentum_factor']
+        self.df['momentum_speed'] = self.df['speed'] * self.df['momentum_factor']
+        self.df['momentum_zomma'] = self.df['zomma'] * self.df['momentum_factor']
+        self.df['momentum_color'] = self.df['color'] * self.df['momentum_factor']
+        self.df['momentum_ultima'] = self.df['ultima'] * self.df['momentum_factor']
+
+
+
+
+        self.df['weighted_liquidity'] = (
+            (self.df['bid_size'] + self.df['ask_size']) / (self.df['ask'] - self.df['bid']) * self.df['oi']
+        )
+        self.df['relative_iv'] = self.df['iv'] / self.df['vwap']  # Assuming vwap as a proxy for historical volatility
+        self.df['gamma_exposure'] = self.df['gamma'] * self.df['oi']
+        
+
+        # Calculate returns as the percentage change in the underlying price
+        self.df['returns'] = self.df['underlying_price'].pct_change()
+
+        # Calculate skewness of returns
+        self.df['returns_skewness'] = self.df['returns'].rolling(window=20).apply(stats.skew, raw=True)
+
+        # Calculate skewness of implied volatility
+        self.df['iv_skewness'] = self.df['iv'].rolling(window=20).apply(stats.skew, raw=True)
+
+
+        # Separate DataFrames for calls and puts
+        self.call_positioning = self.df[self.df['cp'] == 'call']
+        self.put_positioning = self.df[self.df['cp'] == 'put']
+
+
+        # Aggregate dealer positioning metrics for calls, incorporating volume and OI
+        self.call_dealer_positioning = self.call_positioning.groupby('strike').agg({
+            'weighted_delta': 'sum',
+            'weighted_gamma': 'sum',
+            'weighted_vega': 'sum',
+            'weighted_theta': 'sum',
+            'weighted_vanna': 'sum',
+            'weighted_charm': 'sum',
+            'weighted_veta': 'sum',
+            'weighted_speed': 'sum',
+            'weighted_zomma': 'sum',
+            'weighted_color': 'sum',
+            'weighted_ultima': 'sum',
+            'weighted_oi': 'sum',
+            'oi': 'sum',
+            'vol': 'sum'
+        }).reset_index()
+
+        # Normalize by total open interest to get true positioning
+        for column in ['weighted_delta', 'weighted_gamma', 'weighted_vega', 'weighted_theta', 'weighted_vanna',
+                    'weighted_charm', 'weighted_veta', 'weighted_speed', 'weighted_zomma', 'weighted_color',
+                    'weighted_ultima', 'weighted_oi']:
+            self.call_dealer_positioning[column] = self.call_dealer_positioning[column] / self.call_dealer_positioning['oi']
+
+        # Repeat the same for puts
+        self.put_dealer_positioning = self.put_positioning.groupby('strike').agg({
+            'weighted_delta': 'sum',
+            'weighted_gamma': 'sum',
+            'weighted_vega': 'sum',
+            'weighted_theta': 'sum',
+            'weighted_vanna': 'sum',
+            'weighted_charm': 'sum',
+            'weighted_veta': 'sum',
+            'weighted_speed': 'sum',
+            'weighted_zomma': 'sum',
+            'weighted_color': 'sum',
+            'weighted_ultima': 'sum',
+            'weighted_oi': 'sum',
+            'oi': 'sum',
+            'vol': 'sum'
+        }).reset_index()
+
+        for column in ['weighted_delta', 'weighted_gamma', 'weighted_vega', 'weighted_theta', 'weighted_vanna',
+                    'weighted_charm', 'weighted_veta', 'weighted_speed', 'weighted_zomma', 'weighted_color',
+                    'weighted_ultima', 'weighted_oi']:
+            self.put_dealer_positioning[column] = self.put_dealer_positioning[column] / self.put_dealer_positioning['oi']
+
+    def save_conversations_to_jsonl(self, filename: str):
+        """
+        Save the options data as a JSONL file formatted for fine-tuning.
+
+        :param filename: The name of the file to save the data to (e.g., 'options_data.jsonl').
+        """
+        conversations = []
+
+        for index, row in self.df.iterrows():
+            conversation = {
+                "messages": [
+                    {
+                        "role": "system",
+                        "content": "You are an assistant specialized in financial data analysis and options trading."
+                    },
+                    {
+                        "role": "user",
+                        "content": f"Can you analyze the following option data?\n\n"
+                                   f"Strike: {row['strike']}, Expiry: {row['expiry']}, Contract Type: {row['cp']}, "
+                                   f"Open Interest: {row['oi']}, Implied Volatility: {row['iv']}, Delta: {row['delta']}, "
+                                   f"Gamma: {row['gamma']}, Theta: {row['theta']}, Vega: {row['vega']}, "
+                                   f"Moneyness: {row['moneyness']}, Time Value: {row['time_value']}, "
+                                   f"Intrinsic Value: {row['intrinsic_value']}, Extrinsic Value: {row['extrinsic_value']}."
+                    },
+                    {
+                        "role": "assistant",
+                        "content": f"The option is {row['moneyness']} with a strike price of {row['strike']} and "
+                                   f"an expiration date of {row['expiry']}. The current implied volatility is "
+                                   f"{row['iv']}, and the delta is {row['delta']}, indicating that the option's "
+                                   f"price will move {row['delta'] * 100:.2f}% for every 1% change in the underlying asset's price."
+                    }
+                ]
+            }
+            conversations.append(conversation)
+
+        # Save the conversations as a JSONL file
+        with open(filename, 'w') as file:
+            for conversation in conversations:
+                file.write(json.dumps(conversation) + '\n')
+    def get_call_dealer_positioning(self):
+        """
+        Method to get call dealer positioning as a DataFrame.
+        """
+        return self.call_dealer_positioning
+
+    def get_put_dealer_positioning(self):
+        """
+        Method to get put dealer positioning as a DataFrame.
+        """
+        return self.put_dealer_positioning
+    def __repr__(self) -> str:
+        return f"UniversalOptionSnapshot(break_even={self.break_even}, \
+                implied_volatility={self.implied_volatility},\
+                open_interest ={self.open_interest}, \
+                change={self.exchange}, \
+                expiry={self.expiry}, \
+                ticker={self.ticker} \
+                contract_type={self.contract_type}, \
+                exercise_style={self.exercise_style}, \
+                theta={self.theta}, \
+                delta={self.delta}, \
+                gamma={self.gamma}, \
+                vega={self.vega}, \
+                sip_timestamp={self.sip_timestamp}, \
+                conditions={self.conditions}, \
+                trade_price={self.trade_price}, \
+                trade_size={self.trade_size}, \
+                exchange={self.exchange}, \
+                ask={self.ask}, \
+                bid={self.bid}, \
+                bid_size={self.bid_size}, \
+                ask_size={self.ask_size}, \
+                midpoint={self.midpoint}, \
+                change_to_breakeven={self.change_to_breakeven}, \
+                underlying_price={self.underlying_price}, \
+                underlying_ticker={self.underlying_ticker})"
+
+    def __getitem__(self, index):
+        return self.df[index]
+
+    def __setitem__(self, index, value):
+        self.df[index] = value
+
+    def __iter__(self):
+        # If df is a DataFrame, it's already iterable (over its column labels)
+        # To iterate over rows, use itertuples or iterrows
+        self.iter = self.df.itertuples()
+        return self
+
+    def __next__(self):
+        # Just return the next value from the DataFrame iterator
+        try:
+            return next(self.iter)
+        except StopIteration:
+            # When there are no more rows, stop iteration
+            raise StopIteration
 
 class CallsOrPuts:
     def __init__(self, data):
