@@ -20,12 +20,12 @@ from _markets.list_sets.ticker_lists import most_active_tickers
 import requests
 discord = DiscordSDK()
 from discord_webhook import AsyncDiscordWebhook, DiscordEmbed
-opts = PolygonOptions(user='chuck', database='markets')
+opts = PolygonOptions(user='chuck', database='market_data')
 import os
 import aiohttp
 from dotenv import load_dotenv
 load_dotenv()
-poly = Polygon(host='localhost', user='chuck', database='markets', port=5432, password='fud')
+poly = Polygon(host='localhost', user='chuck', database='market_data', port=5432, password='fud')
 
 
 from tabulate import tabulate
@@ -42,7 +42,7 @@ class Technicals:
         self.discord = discord
         self.poly = poly
         self.sema = asyncio.Semaphore(3)
-        self.db = PolygonDatabase(host='localhost', user='chuck', port=5432, password='fud', database='markets')
+        self.db = PolygonDatabase(host='localhost', user='chuck', port=5432, password='fud', database='market_data')
         self.macd_webhooks =  {
             'minute': os.environ.get('macd_minute'),
             'hour': os.environ.get('macd_hour'),
