@@ -2,15 +2,18 @@ import os
 connection_string = os.environ.get('CONNECTION_STRING')
 import aiohttp
 import asyncio
+import httpx
 import pandas as pd
+from fudstop.apis.webull.webull_option_screener import WebullOptionScreener
 from pytz import timezone
+from .industry import IndustryData
 from .database_manager import DatabaseManager
-
+from .futures import WBFutures, IndividualFutures
 from .webull_helpers import parse_most_active, parse_total_top_options, parse_contract_top_options, parse_ticker_values, parse_ipo_data, parse_etfs
-
+screen = WebullOptionScreener()
 class WebullMarkets(DatabaseManager):
     """General market data from webull"""
-    def __init__(self, host,user,port,password,database):
+    def __init__(self,host='localhost', user='chuck', database='market_data', password='fud', port=5432):
 
         self.host=host
         self.user=user
@@ -24,7 +27,36 @@ class WebullMarkets(DatabaseManager):
         self.etf_types = ['industry', 'index', 'commodity', 'other']
         self.high_and_low_types = ['newHigh', 'newLow', 'nearHigh', 'nearLow']
 
-       
+        self.futures_categories = {
+                    'micro': {
+                        'MES': 'E-Mini SP500',
+                        'MNQ': 'E-Mini Nasdaq',
+                        'M2K': 'E-Mini Russell',
+                        'MGC': 'Gold',
+                        'MHG': 'Copper',
+                        'SIL': 'Silver',
+                        'MCL': 'Crude Oil',
+                        '10Y': '10-Year Yield',
+                        '30Y': '30-Year Yield',
+                        '2YY': '2-Year Yield',
+                        '5YY': '5-Year Yield',
+                        'MBT': 'Bitcoin',
+                        'MET': 'Etherium'
+                    },
+                    'equityIndex': {
+                        'YM': 'Dow',
+                        'NQ': 'Nasdaq',
+                        'ES': 'E-Mini SP500'
+                    },
+                    'interest': {
+                        'ZB': 'U.S. Treasury Bond',
+                        'UB': 'Ultra T-Bond'
+                    },
+                    'energy': {
+                        'CL': 'Crude Oil Main',
+                        'BZ': 'Brent Crude Oil'
+                    }}
+
 
         self.headers = {
         "App": "global",
@@ -128,6 +160,75 @@ class WebullMarkets(DatabaseManager):
         return df
 
 
+    async def get_all_futures(self, futures_type):
+        """Get futures data for a given type"""
+        if futures_type not in self.futures_categories:
+            print(f"Futures type '{futures_type}' not found.")
+            return
+
+        async with httpx.AsyncClient() as client:
+            tasks = []
+            futures_dict = self.futures_categories[futures_type]
+            for abbreviation in futures_dict.keys():
+                endpoint = f"https://quotes-gw.webullfintech.com/api/wlas/ranking/futures?regionId=6&rankType={futures_type}.{abbreviation}&pageIndex=1&pageSize=50"
+                print(endpoint)
+                tasks.append(self.fetch_futures_data(client, endpoint, futures_type, abbreviation))
+
+            results = await asyncio.gather(*tasks)
+
+            data = [i.get('data') for i in results]
+            data = [i.get('data') for i in data]
+            data = [item for sublist in data for item in sublist]
+
+            values = [i.get('values') for i in data]
+            futures = [i.get('futures') for i in data]
+
+            return WBFutures(values, futures)
+        
+
+    async def get_equity_index_futures(self, abbreviation):
+        """Get futures data for a given type"""
+
+        async with httpx.AsyncClient() as client:
+
+            endpoint = f"https://quotes-gw.webullfintech.com/api/wlas/ranking/futures?regionId=6&rankType=equityIndex.{abbreviation}&pageIndex=1&pageSize=50"
+
+            data = await client.get(endpoint)
+            results = data.json()
+
+
+            data = results['data']
+            values = [i.get('values') for i in data]
+            futures = [i.get('futures') for i in data]
+
+            return WBFutures(values, futures)
+        
+
+    async def get_industry_data(self, level:int=2, direction:int=-1, timespan:str='5d', page_size:int=100,top_num:int=100):
+        endpoint = f"https://quotes-gw.webullfintech.com/api/wlas/industry/IndustryList"
+        headers = self.headers
+        payload = {"industryLevel":level,"direction":direction,"industryType":timespan,"pageIndex":1,"pageSize":page_size,"regionId":6,"topNum":top_num}
+        async with httpx.AsyncClient() as client:
+            data = await client.post(endpoint, json=payload, headers=headers)
+            data = data.json()
+
+            return IndustryData(data)
+
+    async def get_es_main(self):
+
+        """GET ES MAIN DATA"""
+        endpoint = f"https://quotes-gw.webullfintech.com/api/bgw/quote/realtime?ids=470004426&includeSecu=1&includeQuote=1&more=1"
+        async with httpx.AsyncClient() as client:
+            data = await client.get(endpoint)
+            data = data.json()
+
+            return IndividualFutures(data)
+
+    async def fetch_futures_data(self, client, endpoint, futures_type, abbreviation):
+        response = await client.get(endpoint)
+        data = response.json()
+        return { 'type': futures_type, 'abbreviation': abbreviation, 'data': data }
+
     async def get_most_active(self, rank_type:str='rvol10d', as_dataframe:bool=False):
         """Rank types: 
         
