@@ -8,7 +8,7 @@ import uuid
 from pytz import timezone
 from asyncpg.exceptions import UniqueViolationError
 class DBManager:
-    def __init__(self, host='localhost', user='chuck', password='fud', database='markets', port=5432):
+    def __init__(self, host='localhost', user='chuck', password='fud', database='market_data', port=5432):
         self.db_params = {
             'host': host,
             'user': user,
