@@ -252,7 +252,15 @@ class OptionData:
             # 'profit_potential': self.opp,
             'dte': self.days_to_expiry,
             'time_value': self.time_value,
-            'moneyness': self.moneyness
+            'moneyness': self.moneyness,
+            'vomma': self.vomma,
+            'vanna': self.vanna,
+            'color': self.color,
+            'speed': self.speed,
+            'veta': self.veta,
+            'charm': self.charm,
+            'zomma': self.zomma,
+            'ultima': self.ultima,
 
 
 
