@@ -27,10 +27,10 @@ import pandas as pd
 class OptionsCOG(commands.Cog):
     def __init__(self, bot):
         self.bot=bot
-        self.poly = PolygonOptions(user='chuck', database='markets', host='localhost', port=5432, password='fud')
+        self.poly = PolygonOptions(user='chuck', database='market_data', host='localhost', port=5432, password='fud')
         self.wt = WebullTrading()
-        self.wb = WebullOptions(user='chuck', database='markets', host='localhost', password='fud', port=5432)
-        self.db = PolygonDatabase(user='chuck', database='markets', host='localhost', password='fud', port=5432)
+        self.wb = WebullOptions(user='chuck', database='market_data', host='localhost', password='fud', port=5432)
+        self.db = PolygonDatabase(user='chuck', database='market_data', host='localhost', password='fud', port=5432)
         self.ai = OpenAISDK()
 
 
