@@ -1,7 +1,7 @@
 from apis.polygonio.async_polygon_sdk import Polygon
 import os
 import asyncio
-poly = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
+poly = Polygon(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
 
 
 # Creating a database configuration dictionary using os.environ.get for environment variables
