@@ -3,7 +3,7 @@ import os
 
 import asyncio
 
-poly = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
+poly = Polygon(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
 
 
 # Creating a database configuration dictionary using os.environ.get for environment variables
@@ -11,7 +11,7 @@ db_config = {
     "DB_HOST": os.environ.get('DB_HOST', 'localhost'),
     "DB_PASSWORD": os.environ.get('DB_PASSWORD'),
     "DB_NAME": os.environ.get('COMPANY_INFO_STRING'),
-    "DB_PORT": os.environ.get('DB_PORT', 5432),
+    "DB_PORT": os.environ.get('DB_PORT', 5423),
     "DB_USER": os.environ.get('DB_USER')
 }
 
