@@ -1,6 +1,6 @@
 from apis.polygonio.async_polygon_sdk import Polygon
 
-poly = Polygon(host='localhost', user='chuck', database='markets', password='fud', port=5432)
+poly = Polygon(host='localhost', user='chuck', database='market_data', password='fud', port=5432)
 
 import asyncio
 
